{% extends "form_utils/base.html" %}

{% load form_utils %}

{% block form %}
<fieldset class="nested">
	<legend>All forms</legend>

	<fieldset class="left-column nested">
		<legend>Single form</legend>
		{% form form1 %}
		<div class="form-row">
			<button type="submit" data-loading-text="Saving ..." name="action" value="form1">Save single form</button>
		</div>
	</fieldset>

	<fieldset class="right-column nested">
		<legend>Form group</legend>
		{% form form2 %}
		{% form form3 %}
		<div class="form-row">
			<button type="submit" data-loading-text="Saving ..." name="action" value="group1">Save form group</button>
		</div>
	</fieldset>
</fieldset>
{% endblock %}

{% block submit %}
	<div class="submit-row">
		<div class="positive">
			<button type="submit" data-loading-text="Saving ...">Save all forms</button>
		</div>
	</div>
{% endblock %}
