{% extends "pjax_page.html" %}

{% load staticfiles %}

{% block content %}
{{ block.super }}
<h2>POST</h2>
<form action="{{ request.path }}" method="post" class="ajaxform">
	{% csrf_token %}
	{% include "includes/form_loop.html" %}
	<div class="submit-row">
		<div class="positive">
			<button type="submit" data-loading-text="Sending ...">Show</button>
		</div>
	</div>
</form>
{% endblock %}

{% block extrajs %}
	{{ block.super }}
	<script src="{% static "js/ajaxform_init.js" %}"></script>
{% endblock %}
