<ul>
	<li><a href="{% url "utils" %}">Simple javascript utility</a></li>
	<li><a href="{% url "ajaxform" %}">Ajax form</a></li>
	<li><a href="{% url "messages" %}">Messages</a></li>
	<li><a href="{% url "pjax" %}">PJAX (Pushstate AJAX)</a></li>
	<li><a href="{% url "urlpatterns" %}">Urlpatterns</a></li>
	<li><a href="{% url "form_utils" %}">Form utils</a></li>
	<li><a href="{% url "form_utils_foundation" %}">Form utils with foundation</a></li>
	<li><a href="{% url "formset" %}">Formset view</a></li>
	<li><a href="{% url "multiforms" %}">Multiforms view</a></li>
</ul>
