{% extends "base.html" %}

{% load staticfiles %}

{% block head_title %}Ajax form{% endblock %}
{% block site_title %}Ajax form{% endblock %}

{% block content %}
	<form class="ajaxform" action="{{ request.path }}" method="post">
		{% csrf_token %}
		{% include "includes/form_loop.html" %}
		<div class="submit-row">
			<div class="positive">
				<button type="submit" data-loading-text="Saving ...">Save</button>
			</div>
		</div>
	</form>
{% endblock %}

{% block extrajs %}
	{{ block.super }}
	<script src="{% static "js/ajaxform_init.js" %}"></script>
{% endblock %}
