{% extends "base.html" %}

{% load staticfiles %}

{% block head_title %}PJAX (Pushstate AJAX){% endblock %}
{% block site_title %}PJAX (Pushstate AJAX){% endblock %}

{% block content %}
	<ul>
		<li><a href="{% url "pjax" %}">PJAX (Pushstate AJAX) - index</a></li>
		<li><a href="{% url "pjax_messages" %}">Messages</a></li>
		<li><a href="{% url "pjax_form_post" %}">Forms (POST)</a></li>
		<li><a href="{% url "pjax_form_get" %}">Forms (GET)</a></li>
	</ul>
{% endblock %}

{% block extrajs %}
	<script src="{% static "django_ajax_utils/js/pjax.js" %}"></script>
	<script src="{% static "js/pjax_init.js" %}"></script>
{% endblock %}
